/**
 * ReSolEvo09
 *
 * Center for Environmental Systems Research, Kassel
 * Created by Sascha Holzhauer on 25.05.2010
 */
package de.cesr.resolevo;



import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;

import de.cesr.lara.components.LaraBehaviouralOption;
import de.cesr.lara.components.LaraDecider;
import de.cesr.lara.components.LaraDeciderFactory;
import de.cesr.lara.components.LaraDecisionBuilder;
import de.cesr.lara.components.LaraDecisionTree;
import de.cesr.lara.components.LaraGoal;
import de.cesr.lara.components.impl.Log4jLogger;
import de.cesr.lara.components.impl.decision.AbstractLBinaryDecisionLeave;
import de.cesr.lara.components.impl.decision.AbstractLBinaryDecsionTree;
import de.cesr.lara.components.impl.decision.LMatrixDeciderFactory;
import de.cesr.lara.components.impl.decision.LTreeDecider;
import de.cesr.resolevo.ReSolEvoModel.MaximizePayoffGoal;
import de.cesr.resolevo.agents.ReSoLEvoPlayDecisionBuilder;
import de.cesr.resolevo.agents.ReSolEvoAgent;
import de.cesr.resolevo.bos.AvoidSupportGame;
import de.cesr.resolevo.bos.PlaySupportGame;



/**
 * ReSolEvo09
 * 
 * @author Sascha Holzhauer
 * @date 25.05.2010
 * 
 */
public class PlayDecisionTreeFactory implements LaraDeciderFactory<ReSolEvoAgent, LaraBehaviouralOption<ReSolEvoAgent>> {

	static Map<Class<? extends LaraGoal>, Double>									utilities	= new HashMap<Class<? extends LaraGoal>, Double>();
	{
		utilities.put(MaximizePayoffGoal.class, Double.MAX_VALUE);
	}

	static AbstractLBinaryDecsionTree<ReSolEvoAgent, ReSoLEvoPlayDecisionBuilder>	root		= null;

	/**
	 * Logger
	 */
	static private Logger															logger		= Log4jLogger
																										.getLogger(LMatrixDeciderFactory.class);

	static PlayDecisionTreeFactory													factory		= null;

	private PlayDecisionTreeFactory() {
	}

	/**
	 * @return an instance of this factory
	 */
	public static LaraDeciderFactory<ReSolEvoAgent, LaraBehaviouralOption<ReSolEvoAgent>> getFactory() {
		if (factory == null) {
			factory = new PlayDecisionTreeFactory();
		}
		return factory;
	}

	/**
	 * @return the decision tree's root Created by Sascha Holzhauer on 25.05.2010
	 */
	public static LaraDecisionTree<ReSolEvoAgent, ReSoLEvoPlayDecisionBuilder> getDecisionTree() {
		if (root == null) {
			root = new AbstractLBinaryDecsionTree<ReSolEvoAgent, ReSoLEvoPlayDecisionBuilder>() {
				@Override
				protected boolean evaluate(ReSolEvoAgent agent, ReSoLEvoPlayDecisionBuilder dBuilder) {
					return (!(dBuilder).isInitiated())
							|| ((dBuilder).isInitiated() && agent.getGamePartner(dBuilder).interacts(agent));
				}
			};
			root.setTrueDecisionTree(new AbstractLBinaryDecsionTree<ReSolEvoAgent, ReSoLEvoPlayDecisionBuilder>(
					new AbstractLBinaryDecisionLeave<ReSolEvoAgent, ReSoLEvoPlayDecisionBuilder>() {
						@Override
						public Set<LaraBehaviouralOption> getBos(ReSolEvoAgent agent,
								ReSoLEvoPlayDecisionBuilder dBuilder) {
							Set<LaraBehaviouralOption> bos = new HashSet<LaraBehaviouralOption>(1);
							bos.add(new PlaySupportGame("Cooperate", agent, utilities, agent.getGamePartner(dBuilder),
									true));
							return bos;
						}
					},
					new AbstractLBinaryDecisionLeave<ReSolEvoAgent, ReSoLEvoPlayDecisionBuilder>() {
						@Override
						public Set<LaraBehaviouralOption> getBos(ReSolEvoAgent agent,
								ReSoLEvoPlayDecisionBuilder dBuilder) {
							Set<LaraBehaviouralOption> bos = new HashSet<LaraBehaviouralOption>(1);
							bos.add(new PlaySupportGame("Defect", agent, utilities, agent.getGamePartner(dBuilder),
									false));
							return bos;
						}
					}) {
				@Override
				protected boolean evaluate(ReSolEvoAgent agent, ReSoLEvoPlayDecisionBuilder dBuilder) {
					return agent.getReceivingAdviceGain(agent.getGamePartner(dBuilder))
							- agent.getGivingAdviseLoss(agent.getGamePartner(dBuilder)) >= 0.0;
				}
			});

			root.setFalseDecisionTree(new AbstractLBinaryDecisionLeave<ReSolEvoAgent, ReSoLEvoPlayDecisionBuilder>() {
				@Override
				public Set<LaraBehaviouralOption> getBos(ReSolEvoAgent agent, ReSoLEvoPlayDecisionBuilder dBuilder) {
					// no action required - empty set returned:
					Set<LaraBehaviouralOption> bos = new HashSet<LaraBehaviouralOption>(1);
					bos.add(new AvoidSupportGame("Avoid", agent, utilities, agent.getGamePartner(dBuilder)));
					return bos;
				}
			});
		}
		return root;
	}

	@Override
	public LaraDecider getDecider(ReSolEvoAgent agent, LaraDecisionBuilder dBuilder) {
		return new LTreeDecider<ReSolEvoAgent, ReSoLEvoPlayDecisionBuilder>(getDecisionTree(), agent,
				(ReSoLEvoPlayDecisionBuilder) dBuilder);
	}
}