/**
 * This file is part of
 * 
 * Replicating Solidarity Evolution (ReSolEvo) 0.9
 *
 * Copyright (C) 2010 Center for Environmental Systems Research, Kassel, Germany
 * 
 * ReSolEvo is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * ReSolEvo is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Created by Sascha Holzhauer on 07.01.2010
 */
package de.cesr.resolevo;



import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import org.apache.log4j.Logger;
import org.apache.tools.ant.taskdefs.Sleep;

import repast.simphony.parameter.IllegalParameterException;
import repast.simphony.random.RandomHelper;
import sun.util.logging.resources.logging;
import de.cesr.lara.components.LaraBOMemory;
import de.cesr.lara.components.LaraBOScanner;
import de.cesr.lara.components.LaraDecisionBuilder;
import de.cesr.lara.components.LaraGoal;
import de.cesr.lara.components.impl.Log4jLogger;
import de.cesr.resolevo.ReSolEvoModel.MaximizePayoffGoal;
import de.cesr.resolevo.ReSolEvoParameterManager.Parameter;
import de.cesr.resolevo.agents.ReSolEvoAgent;
import de.cesr.resolevo.bos.AskKnownAgent;
import de.cesr.resolevo.bos.ExploreAgent;
import de.cesr.resolevo.bos.SelectAgent;
import de.cesr.resolevo.util.ReSolEvoParameterDB;



/**
 * 
 * @author Sascha Holzhauer
 * @date 25.02.2010
 * 
 */
public class ReSolEvoAgentSelectionBORecaller implements LaraBOScanner<ReSolEvoAgent, SelectAgent> {
	
	/**
	 * Logger
	 */
	static private Logger logger = Log4jLogger.getLogger(ReSolEvoAgentSelectionBORecaller.class);

	/**
	 * @see de.cesr.lara.components.LaraBOScanner#getBOs(de.cesr.lara.components.LaraSimpleAgent, de.cesr.lara.components.LaraBOMemory,
	 *      de.cesr.lara.components.LaraDecisionBuilder)
	 */
	@Override
	public Collection<SelectAgent> getBOs(ReSolEvoAgent agent, LaraBOMemory<SelectAgent> memory,
			LaraDecisionBuilder dBuilder) {

		Set<SelectAgent> bos = new TreeSet<SelectAgent>();

		if (!(dBuilder == ReSolEvoModel.AmoreDecision.DBUILDER_SELECTION.getDBuilder())) {
			throw new IllegalStateException("This BO Recaller is only intended for Agent Selection decision making");
		}

		Set<SelectAgent> recentBOs = memory.recallAllMostRecent();
		// TODO check for name instead of identity!
		Map<String, SelectAgent> usedBos =  agent.getUsedBOs();
		Collection<SelectAgent> toRemove = new ArrayList<SelectAgent>();
		for (SelectAgent selectRecent : recentBOs) {
			if (usedBos.containsKey(selectRecent.getKey())) {
				toRemove.add(selectRecent);
			}
		}
		for (SelectAgent selectAgent : toRemove) {
			recentBOs.remove(selectAgent);
		}

		if (logger.isDebugEnabled()) {
			logger.debug("noticed: " + agent.getNumNoticed() + " / all: " + agent.getAllNeigbours());
			logger.debug("removed: " + agent.getUsedBOs() + " / remeining: " + recentBOs);
		}
		
		// See Pujol2005, section 2.18:
		for (int i = 0; i < ((Integer) ReSolEvoParameterDB.getInstance().getParameter(Parameter.NUM_INTERACTIONS_AT_ONCE))
				.intValue(); i++) {
			double rand = RandomHelper.nextDouble();
			if (rand < calculateExploreProbability(agent)) {
				ReSolEvoAgent partner = null;

				// UNDO
				ReSolEvoModel.increasenumExploreds();
//				if (logger.isDebugEnabled()) {
//					logger.debug("Random: " + rand + " / prob.: "+  calculateExploreProbability(agent));
//				}
				if (logger.isDebugEnabled()) {
					logger.debug("-Exploration!");
				}
					
				switch (((Integer) ReSolEvoParameterDB.getInstance().getParameter(Parameter.EXPLORATION_METHOD))
						.intValue()) {
					case 1: {
						// explore among all memory entries:
						int num2Explore = recentBOs.size();
						int randomExplored = RandomHelper.nextIntFromTo(0, num2Explore - 1);

						SelectAgent selectBo = (SelectAgent) recentBOs.toArray()[randomExplored];
						partner = selectBo.getAlter();
						// to guarantee that the explored partner is selected the MAXIMIZE_PAYOFF_GOAL is maximised:
						Map<Class<? extends LaraGoal>, Double> utilities = new HashMap<Class<? extends LaraGoal>, Double>();
						utilities.put(MaximizePayoffGoal.class, Double.POSITIVE_INFINITY);

						recentBOs.remove(selectBo);
						bos.add(new AskKnownAgent(partner.getAgentID(), agent, utilities, partner, false));
						if (recentBOs.size() == 0) {
							return bos;
						}
						break;
					}
					case 2: {
						// only from noticed partners:
						// TODO make more efficient
						ArrayList<ExploreAgent> neighbours = new ArrayList<ExploreAgent>();
						for (SelectAgent select : bos) {
							if (select instanceof ExploreAgent) {
								neighbours.add((ExploreAgent) select);
							}
						}
						int random = RandomHelper.nextIntFromTo(0, neighbours.size() - 1);
						bos.add(neighbours.get(random));
						recentBOs.remove(neighbours.get(random));

						break;
					}
					default: {
						throw new IllegalParameterException("Parameter for exploration method is not supported");
					}
				}

			}
		}
		// TODO choose only BOs of Type SelectAgent! - at the moment there
		// should be no other type...
		// adds only BOs to the TreeSet that are not already contained:
		bos.addAll(recentBOs);
		if (logger.isDebugEnabled()) {
			logger.debug(agent + "'s Bos: " + bos);
		}
		return bos;
	}

	/**
	 * See Pujol2005, section 2.19
	 * 
	 * @param agent
	 * @return Created by Sascha Holzhauer on 04.03.2010
	 */
	private double calculateExploreProbability(ReSolEvoAgent agent) {
		// <- LOGGING
		if (logger.isDebugEnabled()) {
			logger.debug("Calculate ExpProb: Number of Known: " + (agent.getAllNeigbours() - agent.getNumNoticed()));
		}
		// LOGGING ->

		double e = agent.getAllNeigbours() == 0 ? 1.0 : Math.pow((double) agent.getNumNoticed()
				/ (double) agent.getAllNeigbours(), 2.0);
		assert (e <= 1.0) : "probability value may not exceed 1.0, but was " + e;
		return e;
	}
}
