/**
 * ReSolEvo09
 *
 * Center for Environmental Systems Research, Kassel
 * Created by Sascha Holzhauer on 16.06.2010
 */
package de.cesr.resolevo;



import java.util.Collections;
import java.util.List;
import java.util.Random;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;

import org.apache.log4j.Logger;

import repast.simphony.random.RandomHelper;
import de.cesr.lara.components.LaraDecisionBuilder;
import de.cesr.lara.components.impl.AbstractLModel;
import de.cesr.lara.components.impl.Log4jLogger;
import de.cesr.resolevo.ReSolEvoModel.AmoreDecision;
import de.cesr.resolevo.ReSolEvoParameterManager.Parameter;
import de.cesr.resolevo.agents.ReSolEvoAgent;
import de.cesr.resolevo.util.ReSolEvoParameterDB;



/**
 * ReSolEvo09
 * 
 * @author Sascha Holzhauer
 * @date 16.06.2010
 * 
 */
public class ReSolEvoIterativeTriggeringModel extends AbstractLModel {

	/**
	 * Logger
	 */
	static private Logger		logger	= Log4jLogger.getLogger(ReSolEvoIterativeTriggeringModel.class);
	private List<ReSolEvoAgent>	agents;
	private long				lastStep;

	/**
	 * @param agents
	 */
	public ReSolEvoIterativeTriggeringModel() {
		
	}

	public void setAgents(List<ReSolEvoAgent> agents) {
		this.agents = agents;
	}
	
	@Override
	public void process() {
		logger.info("-----> Step " + getCurrentStep() + " <-----");
		try {
			System.out.println("SysOut: -----> Step " + getCurrentStep() + " <----- (last took "
					+ DatatypeFactory.newInstance().newDuration((System.currentTimeMillis() - lastStep)) + ")");
		} catch (DatatypeConfigurationException e) {
			e.printStackTrace();
		}
		lastStep = System.currentTimeMillis();

		Collections.shuffle(agents, new Random(RandomHelper.getSeed()));

		//processConsecutively(AmoreDecision.DBUILDER_SELECTION.getDBuilder());
		LaraDecisionBuilder dBuilder = AmoreDecision.DBUILDER_SELECTION.getDBuilder();
		logger.info("Simulating timestep");
		perceiveEnvironment(dBuilder);
		preprocessBOs(dBuilder);
		//makeDecisions(dBuilder);
		//processDecisions(dBuilder);
		clean(dBuilder);
	}

	@Override
	protected void clean(LaraDecisionBuilder dBuilder) {
		for (ReSolEvoAgent agent : agents) {
			agent.clean(dBuilder);
		}
	}

	@Override
	protected void makeDecisions(LaraDecisionBuilder dBuilder) {
		for (ReSolEvoAgent agent : agents) {
			logger.info(agent + "> decide (" + dBuilder.getId() + ")");
			agent.decide(dBuilder);
		}
	}

	@Override
	protected void perceiveEnvironment(LaraDecisionBuilder dBuilder) {
		// nothing to do
	}

	/**
	 * Combines
	 * -pre-processing
	 * -makeDecision
	 * -processDecision
	 *  
	 * @see de.cesr.lara.components.impl.AbstractLModel#preprocessBOs(de.cesr.lara.components.LaraDecisionBuilder)
	 */
	@Override
	protected void preprocessBOs(LaraDecisionBuilder dBuilder) {
		for (ReSolEvoAgent agent : agents) {
			for (int i = 0; i < ((Integer) ReSolEvoParameterDB.getInstance().getParameter(Parameter.NUM_INTERACTIONS)).intValue(); i++) {
				logger.info(agent + "> decide (" + dBuilder.getId() + ")");
				agent.preprocess(dBuilder);
				agent.decide(dBuilder);
				agent.processDecision(dBuilder);
				//agent.clean(dBuilder);
			}
		}
	}

	@Override
	protected void processDecisions(LaraDecisionBuilder dBuilder) {
		// nothing to do since all processing is done in agent's afterDecide()
	}

	@Override
	public void finish() {
		// nothing to do
	}
}
