/**
 * This file is part of
 * 
 * Replicating Solidarity Evolution (ReSolEvo) 0.9
 *
 * Copyright (C) 2010 Center for Environmental Systems Research, Kassel, Germany
 * 
 * ReSolEvo is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * ReSolEvo is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Created by Sascha Holzhauer on 07.01.2010
 */
package de.cesr.resolevo;



import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Random;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;

import org.apache.log4j.Logger;

import repast.simphony.context.Context;
import repast.simphony.dataLoader.ContextBuilder;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.schedule.ScheduleParameters;
import repast.simphony.engine.schedule.ScheduledMethod;
import repast.simphony.parameter.IllegalParameterException;
import repast.simphony.random.RandomHelper;
import de.cesr.lara.components.LaraDecisionBuilder;
import de.cesr.lara.components.LaraGoal;
import de.cesr.lara.components.LaraModel;
import de.cesr.lara.components.impl.AbstractLModel;
import de.cesr.lara.components.impl.AbstractLaraAgent;
import de.cesr.lara.components.impl.LModel;
import de.cesr.lara.components.impl.Log4jLogger;
import de.cesr.lara.components.impl.decision.LDecisionHeuristicComponent_MaxLineTotalRandomAtTie;
import de.cesr.lara.components.impl.decision.LaraDecisionBuilderImpl;
import de.cesr.resolevo.ReSolEvoParameterManager.Parameter;
import de.cesr.resolevo.agents.ReSoLEvoPlayDecisionBuilder;
import de.cesr.resolevo.agents.ReSolEvoAgent;
import de.cesr.resolevo.bos.AvoidSupportGame;
import de.cesr.resolevo.util.ReSolEvoParameterDB;
import de.cesr.resolevo.util.ReSolEvoParameterXML;



/**
 * 
 * TODO create separate model and agent class for parallel agent triggering mode!
 * 
 * AMoRe - Agents Managing ongoing Relations
 * 
 * @author Sascha Holzhauer
 * @date 06.01.2010
 */
public class ReSolEvoModel extends AbstractLModel implements ContextBuilder<Object> {

	/**
	 * Logger
	 */
	static private Logger					logger			= Log4jLogger.getLogger(ReSolEvoModel.class);

	private static ReSolEvoParameterManager	paraMan			= ReSolEvoParameterXML.getInstance();

	private static long						startTimeMillis;
	private static long						lastStep;

	/**
	 * 
	 */
	private static int						numGamesPlayed	= 0;

	/**
	 * @param numGamesPlayed the numGamesPlayed to set
	 */
	public static void increaseNumGamesPlayed() {
		numGamesPlayed++;
	}

	private static int	numExchangedMemories	= 0;

	/**
	 * @param numGamesPlayed the numGamesPlayed to set
	 */
	public static void increasenumExchangedMmeories() {
		numExchangedMemories++;
	}

	private static int	numUpdatedMems	= 0;

	/**
	 * @param numGamesPlayed the numGamesPlayed to set
	 */
	public static void increasenumUpdatedMems() {
		numUpdatedMems++;
	}

	private static int	numExplored	= 0;

	/**
	 * @param numGamesPlayed the numGamesPlayed to set
	 */
	public static void increasenumExploreds() {
		numExplored++;
	}

	private int				runCounter	= 0;

	private static Context	thisContext;

	private LaraModel		helperModel;

	/**
	 * @author Sascha Holzhauer
	 * @date 04.03.2010
	 * 
	 */
	public enum Network {

		/**
		 * The network of known neighbours
		 */
		NETWORK_KNOWN("Known"),

		/**
		 * The network of neighbours the agents is just aware of but does not know.
		 */
		NETWORK_NOTICED("Noticed");

		private final String	name;

		Network(String n) {
			name = n;
		}

		@Override
		public String toString() {
			return this.name;
		}
	}

	static Collection<Class<? extends LaraGoal>>	goals	= new ArrayList<Class<? extends LaraGoal>>();
	{
		goals.add(MaximizePayoffGoal.class);
	}

	/**
	 * @author Sascha Holzhauer
	 * @date 04.03.2010
	 * 
	 */
	public enum AmoreDecision {
		/**
		 * Partner selection decision
		 */

		DBUILDER_SELECTION(new LaraDecisionBuilderImpl("PartnerSelection",
				new LDecisionHeuristicComponent_MaxLineTotalRandomAtTie()), goals),
		/**
		 * Support game decision
		 */
		DBUILDER_GAME(new LaraDecisionBuilderImpl("SupportGame",
				new LDecisionHeuristicComponent_MaxLineTotalRandomAtTie()), goals);

		LaraDecisionBuilder	dBuilder;

		AmoreDecision(LaraDecisionBuilder dBuilder, Collection<Class<? extends LaraGoal>> goals) {
			dBuilder.setGoals(goals);
			this.dBuilder = dBuilder;
		}

		/**
		 * @return Created by Sascha Holzhauer on 04.03.2010
		 */
		public LaraDecisionBuilder getDBuilder() {
			return dBuilder;
		}
	}

	/**
	 * @uml.associationEnd multiplicity="(1 1)" inverse="a:de.cesr.resolevo.ReSolEvoAgentContext"
	 */
	private ReSolEvoAgentContext	amoreContextBuilder	= new de.cesr.resolevo.ReSolEvoAgentContext();
	private List<ReSolEvoAgent>		agents;

	/**
	 * @author Sascha Holzhauer
	 * @date 22.02.2010
	 * 
	 */
	public class MaximizePayoffGoal implements LaraGoal {
	};

	/**
	 * Invoked when the model is run for the first time step
	 * 
	 * Created by Sascha Holzhauer on 20.04.2010
	 */
	@ScheduledMethod(start = 1, priority = ScheduleParameters.FIRST_PRIORITY)
	public void start() {
		startTimeMillis = System.currentTimeMillis();
		logger.info("Started at " + new SimpleDateFormat("HH:mm:ss:SS").format(new Date(startTimeMillis)));
		System.out.println("Started at " + new SimpleDateFormat("HH:mm:ss:SS").format(new Date(startTimeMillis)));

		agents = new ArrayList<ReSolEvoAgent>();
		for (ReSolEvoAgent agent : ((Context<ReSolEvoAgent>) ReSolEvoModel.thisContext
				.findContext("AMoRePujol2005AgentContext"))) {
			agents.add(agent);
		}
		
		switch (((Integer) paraMan.getParameter(Parameter.AGENT_TRIGGERING)).intValue()) {
			case 3: {
				((ReSolEvoIterativeTriggeringModel)helperModel).setAgents(agents);
				break;
			}
			default: {
				helperModel = this;
				this.init();
			}
		}
		Collections.<ReSolEvoAgent> sort(agents, ReSolEvoAgent.getComparator());
	}

	/**
	 * 
	 * Created by Sascha Holzhauer on 14.01.2010
	 */
	@ScheduledMethod(start = 1, interval = 1)
	public void stepIt() {
		helperModel.step();
	}

	@Override
	public void process() {
		logger.info("-----> Step " + getCurrentStep() + " <-----");
		try {
			System.out.println("SysOut: -----> Step " + getCurrentStep() + " <----- (last took "
					+ DatatypeFactory.newInstance().newDuration((System.currentTimeMillis() - lastStep)) + ")");
		} catch (DatatypeConfigurationException e) {
			e.printStackTrace();
		}
		lastStep = System.currentTimeMillis();
		Collections.shuffle(agents, new Random(RandomHelper.getSeed()));

		switch (((Integer) paraMan.getParameter(Parameter.AGENT_TRIGGERING)).intValue()) {
			case 1:
			case 2: {
				processConsecutively(AmoreDecision.DBUILDER_SELECTION.getDBuilder());
				processConsecutively(AmoreDecision.DBUILDER_GAME.getDBuilder());
				break;
			}
			default: {
				throw new IllegalParameterException("Parameter for AGENT_TRIGGERING not supported");
			}
		}
	}

	/**
	 * Processes a given decision (perceive, pre-process, make decision). This method may be invoked several times (for each decision)
	 * 
	 * @param dBuilder the {@link LaraDecisionBuilder} that represents the decision to process.
	 * @see de.cesr.lara.components.impl.AbstractLModel#processConsecutively(de.cesr.lara.components.LaraDecisionBuilder)
	 */
	@Override
	public void processConsecutively(LaraDecisionBuilder dBuilder) {
		logger.info("Process decision " + dBuilder.getId());
		switch (((Integer) paraMan.getParameter(Parameter.AGENT_TRIGGERING)).intValue()) {
			case 1: {
				if (dBuilder == ReSolEvoModel.AmoreDecision.DBUILDER_SELECTION.getDBuilder()) {
					perceiveEnvironment(dBuilder);
					preprocessBOs(dBuilder);
					makeDecisions(dBuilder);
					processDecisions(dBuilder);
					clean(dBuilder);
				} else {
					perceiveEnvironment(dBuilder);
					preprocessBOs(dBuilder);
					makeDecisions(dBuilder); // makeDecision contains another switch/case
					clean(dBuilder);
				}
				break;
			}
			case 2: {
				// in case of 3 this is only triggered for DBUILDER_SELECTION (!)
				perceiveEnvironment(dBuilder);
				preprocessBOs(dBuilder);
				makeDecisions(dBuilder);
				processDecisions(dBuilder);
				clean(dBuilder);
				break;
			}
			default: {
				throw new IllegalParameterException("Parameter for AGENT_TRIGGERING not supported");
			}
		}
	}

	/**
	 * @see de.cesr.lara.components.impl.LModel#perceiveEnvironment()
	 */
	@Override
	protected void perceiveEnvironment(LaraDecisionBuilder dBuilder) {
		// nothing to do
	}

	/**
	 * @see de.cesr.lara.components.impl.LModel#preprocessBOs()
	 */
	@Override
	protected void preprocessBOs(LaraDecisionBuilder dBuilder) {
		for (ReSolEvoAgent agent : agents) {
			if (dBuilder == AmoreDecision.DBUILDER_GAME.getDBuilder()) {
				for (LaraDecisionBuilder decisionBuilder : agent.getPlayDecisionsActive()) {
					// in case agent triggering is completely iterative (3) the not initiated game decisions are pre-processed implicitly!
					if (((Integer) paraMan.getParameter(Parameter.AGENT_TRIGGERING)).intValue() != 1
							|| ((ReSoLEvoPlayDecisionBuilder) decisionBuilder).isInitiated()) {
						agent.preprocess(decisionBuilder);
					}
				}
			} else {
				// for SelectAgent BOs:
				agent.preprocess(dBuilder);
			}
		}
	}

	/**
	 * @see de.cesr.lara.components.impl.LModel#makeDecisions(de.cesr.lara.components.LaraDecision)
	 */
	@Override
	protected void makeDecisions(LaraDecisionBuilder dBuilder) {
		if (dBuilder == AmoreDecision.DBUILDER_GAME.getDBuilder()) {
			switch (((Integer) paraMan.getParameter(Parameter.AGENT_TRIGGERING)).intValue()) {
				case 1: {
					for (ReSolEvoAgent agent : agents) {
						for (LaraDecisionBuilder decisionBuilder : agent.getPlayDecisionsActive()) {
							// process only initiated games here since all others are processed implicitly:
							if (((ReSoLEvoPlayDecisionBuilder) decisionBuilder).isInitiated()) {

								ReSolEvoAgent partner = null;

								agent.decide(decisionBuilder);

								// check if partner agent accepted interaction:
								if (!(agent.getDecisionData(decisionBuilder).getBo() instanceof AvoidSupportGame)) {
									// let the partner agent choose
									partner = agent.getGamePartner((ReSoLEvoPlayDecisionBuilder) decisionBuilder);
									ReSoLEvoPlayDecisionBuilder partnerDBuilder = partner.getPassivePlayDecision(agent);
									partner.preprocess(partnerDBuilder);
									partner.decide(partnerDBuilder);
									partner.processDecision(partnerDBuilder);
								}
								agent.processDecision(decisionBuilder);
							}
						}
					}
					break;
				}
				case 2: {
					for (ReSolEvoAgent agent : agents) {
						for (LaraDecisionBuilder decisionBuilder : agent.getPlayDecisionsActive()) {
							agent.decide(decisionBuilder);
						}
					}
					for (ReSolEvoAgent agent : agents) {
						for (LaraDecisionBuilder decisionBuilder : agent.getPlayDecisionsPassive()) {
							agent.decide(decisionBuilder);
						}
					}
					break;
				}
				default: {
					throw new IllegalParameterException("Parameter for AGENT_TRIGGERING not supported");
				}
			}
		} else {
			for (ReSolEvoAgent agent : agents) {
				// for SelectAgent BOs:
				logger.debug(agent + "> decide (" + dBuilder.getId() + ")");
				agent.decide(dBuilder);
			}
		}
	}

	/**
	 * @see de.cesr.lara.components.impl.AbstractLModel#processDecisions(de.cesr.lara.components.LaraDecisionBuilder)
	 */
	@Override
	protected void processDecisions(LaraDecisionBuilder dBuilder) {
		for (ReSolEvoAgent agent : agents) {
			if (dBuilder == AmoreDecision.DBUILDER_GAME.getDBuilder()) {
				for (LaraDecisionBuilder decisionBuilder : agent.getPlayDecisionsActive()) {
					agent.processDecision(decisionBuilder);
				}
				switch (((Integer) paraMan.getParameter(Parameter.AGENT_TRIGGERING)).intValue()) {
					case 2: {
						for (LaraDecisionBuilder decisionBuilder : agent.getPlayDecisionsPassive()) {
							agent.processDecision(decisionBuilder);
						}
						break;
					}
					default: {
						break;
					}
				}
			} else {
				agent.processDecision(dBuilder);
			}
		}
	}

	/**
	 * @see de.cesr.lara.components.LaraModel#finish()
	 */
	public void finish() {
		try {
			System.out.println("finished at "
					+ new SimpleDateFormat("HH:mm:ss:SS").format(new Date(System.currentTimeMillis()))
					+ "(taking "
					+ DatatypeFactory.newInstance().newDuration(
							(System.currentTimeMillis() - ReSolEvoModel.getStartTimeMillis())) + ")");
		} catch (DatatypeConfigurationException e) {
			e.printStackTrace();
		}
//		System.out.println("Number of games played: " + numGamesPlayed);
//		System.out.println("Number of exchanged memories: " + numExchangedMemories);
//		System.out.println("Number of updated memories: " + numUpdatedMems);
//		System.out.println("Number of Explored: " + numExplored);
		System.gc();
	}

	/**
	 * Delegates the context building to <code>ReSolEvoAgentContext</code> to encapsulate context building.
	 * 
	 * @see repast.simphony.dataLoader.ContextBuilder#build(repast.simphony.context.Context)
	 */
	@Override
	public Context<?> build(Context<Object> context) {
		if (RunEnvironment.getInstance().getParameters().getSchema().contains(
				ReSolEvoParameterManager.Parameter.PARAMS_ID.name())) {
			ReSolEvoParameterDB.setParameterID(((Integer) RunEnvironment.getInstance().getParameters().getValue(
					ReSolEvoParameterManager.Parameter.PARAMS_ID.name())).intValue());
		}
		
		logger.info("Used ParamID: " + ((Integer) paraMan.getParameter(Parameter.PARAMS_ID)).intValue());
		
		paraMan.initParameters();
		ReSolEvoNetwork.reset();
		RandomHelper.setSeed(((Integer) paraMan.getParameter(Parameter.PARAMS_RANDOMSEED)).intValue());

		AbstractLaraAgent.resetCounter();
		
		switch (((Integer) paraMan.getParameter(Parameter.AGENT_TRIGGERING)).intValue()) {
			case 3: {
				helperModel = new ReSolEvoIterativeTriggeringModel();
				break;
			}
			default: {
				helperModel = this;
			}
		}
		LModel.setNewModel(helperModel);
		LModel.getModel().getLRandom().setSeed(((Integer) ReSolEvoParameterDB.getInstance().getParameter(Parameter.PARAMS_RANDOMSEED)).intValue());
		RunEnvironment.getInstance().getCurrentSchedule().schedule(
				ScheduleParameters.createAtEnd(ScheduleParameters.LAST_PRIORITY), this, "finish");
		RunEnvironment.getInstance().endAt(((Integer) paraMan.getParameter(Parameter.END_TICK)).intValue());
		RunEnvironment.getInstance().getCurrentSchedule().schedule(this);
		amoreContextBuilder = new de.cesr.resolevo.ReSolEvoAgentContext();
		thisContext = context;
		return context; // this.context;
	}

	/**
	 * @return the startTimeMillis
	 */
	static public long getStartTimeMillis() {
		return startTimeMillis;
	}


	/**
	 * 
	 * Created by Sascha Holzhauer on 01.06.2010
	 * 
	 * @see de.cesr.lara.components.impl.AbstractLModel#clean(de.cesr.lara.components.LaraDecisionBuilder)
	 */
	@Override
	protected void clean(LaraDecisionBuilder dBuilder) {
		for (ReSolEvoAgent agent : agents) {
			agent.clean(dBuilder);
		}
	}
}
