/**
 * AMoRe_Pujol2005
 *
 * Center for Environmental Systems Research, Kassel
 * Created by Sascha Holzhauer on 05.03.2010
 */
package de.cesr.resolevo;



import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;

import de.cesr.lara.components.LaraBOMemory;
import de.cesr.lara.components.LaraBOScanner;
import de.cesr.lara.components.LaraDecisionBuilder;
import de.cesr.lara.components.LaraGoal;
import de.cesr.lara.components.impl.Log4jLogger;
import de.cesr.resolevo.ReSolEvoModel.MaximizePayoffGoal;
import de.cesr.resolevo.agents.ReSolEvoAgent;
import de.cesr.resolevo.agents.ReSoLEvoPlayDecisionBuilder;
import de.cesr.resolevo.bos.AskKnownAgent;
import de.cesr.resolevo.bos.PlaySupportGame;
import de.cesr.resolevo.bos.SelectAgent;



/**
 * Initial
 * 
 * @author Sascha Holzhauer
 * @date 05.03.2010
 * 
 */
public class ReSolEvoPlayBoRecaller implements LaraBOScanner<ReSolEvoAgent, PlaySupportGame> {

	/**
	 * Logger
	 */
	static private Logger logger = Log4jLogger.getLogger(ReSolEvoPlayBoRecaller.class);
	
	@Override
	public Collection<PlaySupportGame> getBOs(ReSolEvoAgent agent, LaraBOMemory<PlaySupportGame> memory,
			LaraDecisionBuilder dB) {

		if (!(dB instanceof ReSoLEvoPlayDecisionBuilder)) {
			throw new IllegalStateException(
					"This BO Recaller is only intended for playing support game decision making");
		}
		ReSoLEvoPlayDecisionBuilder dBuilder = (ReSoLEvoPlayDecisionBuilder) dB;

		Collection<PlaySupportGame> bos = new ArrayList<PlaySupportGame>();
		Map<Class<? extends LaraGoal>, Double> utilities = new HashMap<Class<? extends LaraGoal>, Double>();
		utilities.put(MaximizePayoffGoal.class, Double.MAX_VALUE);

		// substitute by tree decision one day:
		// check interaction only when the game was initiated by this agent:
		if ((!((ReSoLEvoPlayDecisionBuilder) dBuilder).isInitiated())
				|| (((ReSoLEvoPlayDecisionBuilder) dBuilder).isInitiated()
				&& agent.getGamePartner(dBuilder).interacts(agent))) {
			if (agent.getReceivingAdviceGain(agent.getGamePartner(dBuilder))
					- agent.getGivingAdviseLoss(agent.getGamePartner(dBuilder)) >= 0.0) {
				bos.add(new PlaySupportGame("Cooperate", agent, utilities, agent.getGamePartner(dBuilder), true));
			} else {
				bos.add(new PlaySupportGame("Defect", agent, utilities, agent.getGamePartner(dBuilder), false));
			}
		} else {
			// interaction refused (Pseudo code: interactionIsRefused(agent, partner)):
			if (agent.getLaraComp().getBOMemory().contains(agent.getGamePartner(dBuilder).getAgentID())) {
				// parter is known:
				utilities.put(MaximizePayoffGoal.class, new Double(-agent.getGivingAdviseLoss(agent
						.getGamePartner(dBuilder))));
				// UNDO
				logger.warn("in getBOs");
				agent.getLaraComp().getBOMemory().refresh(
						new AskKnownAgent(agent.getGamePartner(dBuilder).getAgentID(), agent, utilities, agent
								.getGamePartner(dBuilder),
						// TODO make optional (important when changing memory capacity management)
								((SelectAgent) agent.getLaraComp().getBOMemory().recall(
										agent.getGamePartner(dBuilder).getAgentID())).isFromME()));
			}
			agent.removeActiveGameDecision((ReSoLEvoPlayDecisionBuilder) dBuilder);
		}
		return bos;
	}
}
