/**
 * This file is part of
 * 
 * Replicating Solidarity Evolution (ReSolEvo) 0.9
 *
 * Copyright (C) 2010 Center for Environmental Systems Research, Kassel, Germany
 * 
 * ReSolEvo is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * ReSolEvo is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Created by Sascha Holzhauer on 07.01.2010
 */
package de.cesr.resolevo.agents;



import java.util.HashSet;
import java.util.Set;

import de.cesr.lara.components.LaraBehaviouralOption;
import de.cesr.lara.components.LaraDeliberativeChoiceComponent;
import de.cesr.lara.components.LaraGoal;
import de.cesr.lara.components.impl.decision.LaraDecisionBuilderImpl;
import de.cesr.resolevo.ReSolEvoModel.MaximizePayoffGoal;



/**
 * AMoRe_Pujol2005
 * 
 * @author Sascha Holzhauer
 * @date 29.04.2010
 * 
 */
public class ReSoLEvoPlayDecisionBuilder extends LaraDecisionBuilderImpl implements Comparable<ReSoLEvoPlayDecisionBuilder> {

	private boolean initiated;
	/**
	 * @param name
	 * @param heuristic
	 * @param initiated true if this game was initiated by the possessing agent
	 */
	public ReSoLEvoPlayDecisionBuilder(String name, LaraDeliberativeChoiceComponent<LaraBehaviouralOption<ReSolEvoAgent>> heuristic,
			boolean initiated) {
		super(name, heuristic);
		this.initiated = initiated;
		Set<Class<? extends LaraGoal>> goals = new HashSet<Class<? extends LaraGoal>>();
		goals.add(MaximizePayoffGoal.class);
		setGoals(goals);
	}
	
	/**
	 * @return true if this game was initiated by the possessing agent
	 * Created by Sascha Holzhauer on 04.05.2010
	 */
	public boolean isInitiated() {
		return this.initiated;
	}

	@Override
	public int compareTo(ReSoLEvoPlayDecisionBuilder o) {
		return getId().equals(o.getId()) ? 
				(initiated ? ( o.isInitiated() ? 0 : 1) :  o.isInitiated() ? -1 : 0)
						: this.getId().compareTo(o.getId());
	}
	
	public String toString() {
		return "PlayDecision " + getId() + "(" + initiated + ")";
	}
}
