/**
 * This file is part of
 * 
 * Replicating Solidarity Evolution (ReSolEvo) 0.9
 *
 * Copyright (C) 2010 Center for Environmental Systems Research, Kassel, Germany
 * 
 * ReSolEvo is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * ReSolEvo is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Created by Sascha Holzhauer on 07.01.2010
 */
package de.cesr.resolevo.bos;



import java.util.HashMap;
import java.util.Map;

import de.cesr.lara.components.LaraBehaviouralOption;
import de.cesr.lara.components.LaraDecisionBuilder;
import de.cesr.lara.components.LaraGoal;
import de.cesr.lara.components.LaraSimpleAgent;
import de.cesr.resolevo.ReSolEvoModel.MaximizePayoffGoal;
import de.cesr.resolevo.agents.ReSolEvoAgent;



/**
 * AMoRe_Pujol2005
 * 
 * @author Sascha Holzhauer
 * @date 05.05.2010
 * 
 */
public class AskKnownAgent extends SelectAgent {

	/**
	 * @param agent
	 * @param alter
	 * @return Created by Sascha Holzhauer on 04.03.2010
	 */
	public static Map<Class<? extends LaraGoal>, Double> getIntialUtilities(ReSolEvoAgent agent,
			ReSolEvoAgent alter) {
		Map<Class<? extends LaraGoal>, Double> utilities = new HashMap<Class<? extends LaraGoal>, Double>();
		utilities.put(MaximizePayoffGoal.class, new Double(0.0));
		return utilities;
	}

	/**
	 * @param key
	 * @param agent
	 * @param alter
	 * @param fromME
	 */
	public AskKnownAgent(String key, ReSolEvoAgent agent, ReSolEvoAgent alter, boolean fromME) {
		super(key, agent, getIntialUtilities(agent, alter), alter, fromME);
	}

	/**
	 * @param key
	 * @param agent
	 * @param utilities
	 * @param alter
	 * @param fromME
	 */
	public AskKnownAgent(String key, ReSolEvoAgent agent, Map<Class<? extends LaraGoal>, Double> utilities,
			ReSolEvoAgent alter, boolean fromME) {
		super(key, agent, utilities, alter, fromME);
	}

	/**
	 * @see de.cesr.lara.components.impl.AbstractBO#getModifiedBO(de.cesr.lara.components.LaraSimpleAgent, java.util.Map)
	 */
	@Override
	public LaraBehaviouralOption<ReSolEvoAgent> getModifiedBO(ReSolEvoAgent agent, Map<Class<? extends LaraGoal>, Double> utilities) {
		return new AskKnownAgent(getKey(), (ReSolEvoAgent) agent, utilities, getAlter(), isFromME());
	}

	/**
	 * @see de.cesr.lara.components.LaraBehaviouralOption#getComputedUtilities(de.cesr.lara.components.LaraDecisionBuilder)
	 */
	@Override
	public Map<Class<? extends LaraGoal>, Double> getComputedUtilities(LaraDecisionBuilder dBuilder) {
		return getValue();
	}

	/**
	 * @see de.cesr.lara.components.impl.AbstractBO#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object o) {
		if (!(o instanceof AskKnownAgent)) {
			return false;
		}
		return super.equals(o) && ((AskKnownAgent) o).getAlter() == this.getAlter();
	}

	/**
	 * @see de.cesr.lara.components.impl.AbstractBO#hashCode()
	 */
	@Override
	public int hashCode() {
		// TODO check if sufficient
		int result = 17;
		return result * 31 + (getAlter() == null ? 0 : getAlter().hashCode());
	}

	/**
	 * @see de.cesr.lara.components.impl.AbstractBO#toString()
	 */
	@Override
	public String toString() {
		return "AskKnownAgent-BO(" + getAlter().getAgentID() + ")";
	}

}
