/**
 * This file is part of
 * 
 * Replicating Solidarity Evolution (ReSolEvo) 0.9
 *
 * Copyright (C) 2010 Center for Environmental Systems Research, Kassel, Germany
 * 
 * ReSolEvo is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * ReSolEvo is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Created by Sascha Holzhauer on 07.01.2010
 */
package de.cesr.resolevo.bos;



import java.util.Map;

import de.cesr.lara.components.LaraBehaviouralOption;
import de.cesr.lara.components.LaraDecisionBuilder;
import de.cesr.lara.components.LaraGoal;
import de.cesr.lara.components.LaraSimpleAgent;
import de.cesr.resolevo.agents.ReSolEvoAgent;



/**
 * @author Sascha Holzhauer
 * @date 22.02.2010
 * 
 */
public class PlaySupportGame extends LaraBehaviouralOption<ReSolEvoAgent> {

	private final ReSolEvoAgent	partnerAgent;

	/**
	 * @return the partnerAgent
	 */
	public ReSolEvoAgent getPartnerAgent() {
		return partnerAgent;
	}

	private boolean	cooperation;

	/**
	 * @return the cooperation
	 */
	public boolean isCooperation() {
		return cooperation;
	}

	/**
	 * @param key
	 * @param agent the agent the BO belongs to
	 * @param utilities
	 * @param partnerAgent the partner agent
	 * @param cooperation true if this BO represents an cooperative behaviour of the focal agent
	 */
	public PlaySupportGame(String key, ReSolEvoAgent agent, Map<Class<? extends LaraGoal>, Double> utilities,
			ReSolEvoAgent partnerAgent, boolean cooperation) {
		super(key, agent, utilities);
		this.partnerAgent = partnerAgent;
		this.cooperation = cooperation;
	}

	@Override
	public LaraBehaviouralOption<ReSolEvoAgent> getModifiedBO(ReSolEvoAgent agent, Map<Class<? extends LaraGoal>, Double> utilities) {
		return new PlaySupportGame(getKey(), agent, utilities, partnerAgent, this.cooperation);
	}

	@Override
	public Map<Class<? extends LaraGoal>, Double> getComputedUtilities(LaraDecisionBuilder dBuilder) {
		return getModifiableUtilities();
	}

	/**
	 * @see de.cesr.lara.components.impl.AbstractBO#toString()
	 */
	@Override
	public String toString() {
		return "PlayGame(" + (isCooperation() ? "cooperate with " : "defect against ") + partnerAgent.getAgentID()
				+ ")";
	}

	/**
	 * @see de.cesr.lara.components.impl.AbstractBO#hashCode()
	 */
	@Override
	public int hashCode() {
		// TODO check if sufficient
		int result = 17;
		return result * 31 + (partnerAgent == null ? 0 : partnerAgent.hashCode());
	}
}
