/**
 * This file is part of
 * 
 * Replicating Solidarity Evolution (ReSolEvo) 0.9
 *
 * Copyright (C) 2010 Center for Environmental Systems Research, Kassel, Germany
 * 
 * ReSolEvo is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * ReSolEvo is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Created by Sascha Holzhauer on 07.01.2010
 */
package de.cesr.resolevo.util;

import java.awt.BorderLayout;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.Iterator;

import javax.swing.JFrame;

import org.apache.log4j.Logger;

import repast.simphony.context.Context;
import repast.simphony.context.space.graph.ContextJungNetwork;
import repast.simphony.engine.controller.NullAbstractControllerAction;
import repast.simphony.engine.environment.GUIRegistryType;
import repast.simphony.engine.environment.RunEnvironmentBuilder;
import repast.simphony.engine.environment.RunState;
import repast.simphony.engine.schedule.IAction;
import repast.simphony.engine.schedule.NonModelAction;
import repast.simphony.engine.schedule.ScheduleParameters;
import repast.simphony.parameter.Parameters;
import repast.simphony.scenario.ModelInitializer;
import repast.simphony.scenario.Scenario;
import repast.simphony.space.projection.Projection;
import repast.simphony.visualization.IDisplay;
import repast.simphony.visualization.IDisplay.LayoutFrequency;
import repast.simphony.visualization.engine.DisplayDescriptor;
import de.cesr.resolevo.ReSolEvoModel;
import de.cesr.resolevo.agents.ReSolEvoAgent;
import edu.uos.sh.soneta.core.change.ChangeFiring;
import edu.uos.sh.soneta.gui.AbstractNodeTable;
import edu.uos.sh.soneta.gui.DefaultNodeTable;
import edu.uos.sh.soneta.measures.NetworkMeasureSupport;
import edu.uos.sh.soneta.measures.NetworkMeasureUtilities;
import edu.uos.sh.soneta.sadapted.visualization.engine.DefaultDisplayDescriptorAdapted;
import edu.uos.sh.soneta.sadapted.visualization.engine.DisplayProducerAdapted;
import edu.uos.sh.soneta.sadapted.visualization.visualization2D.Display2DAdapted;
import edu.uos.sh.soneta.util.Log4jLogger;

/**
 * 
 * @author Sascha Holzhauer
 * @date 30.03.2010
 * 
 */
public class NetworkDisplayInitializer implements ModelInitializer {
	
	/**
	 * Logger
	 */
	static private Logger logger = Log4jLogger.getLogger(NetworkDisplayInitializer.class);

	private IDisplay display;
	private DefaultDisplayDescriptorAdapted<ReSolEvoAgent, ReSolEvoAgent> descriptor;

	@NonModelAction
	static class DisplayUpdater implements IAction {

		private IDisplay display;

		public DisplayUpdater(IDisplay display) {
			this.display = display;
		}

		public void execute() {
			logger.debug("Execute()");
			display.update();
			display.render();
		}
	}

	/**
	 * This is ran after the model has been loaded. This is only ran once, but
	 * the settings set through the {@link repast.simphony..scenario.Scenario}
	 * will apply to every run of the simulation.
	 * 
	 * @param scen
	 *            the {@link repast.simphony..scenario.Scenario} object that
	 *            hold settings for the run
	 */
	public void initialize(Scenario scen, RunEnvironmentBuilder builder) {

		scen.addMasterControllerAction(new NullAbstractControllerAction() {
			@Override
			public void runInitialize(RunState runState, Context context,
					Parameters runParams) {
				logger.debug("Initialized");

				try {
					DefaultDisplayDescriptorAdapted<ChangeFiring, NetworkMeasureSupport> descriptor = getDescriptor(context);
					display = new DisplayProducerAdapted<ChangeFiring, NetworkMeasureSupport>(
							getDescriptor(context)).createDisplay();
					display.init();
					if (descriptor.isDisplayNodeTable()) {
						setupNodeTable();
					}
				} catch (IllegalAccessException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (InvocationTargetException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (InstantiationException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (ClassNotFoundException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				runState.getGUIRegistry().addDisplay("Network of Knowns",
						GUIRegistryType.OTHER, display);
				runState.getScheduleRegistry().getModelSchedule().schedule(
						ScheduleParameters.createRepeating(1, 1,
								ScheduleParameters.END),
						new DisplayUpdater(display));

			}

			@Override
			public void runCleanup(RunState runState, Context context) {
				display.destroy();
				display = null;
			}

			@Override
			public String toString() {
				return "Network of Knowns";
			}
		});

	}
	
	/**
	 * TODO test!
	 * @param context
	 * @param name
	 * @return
	 * Created by Sascha Holzhauer on 27.04.2010
	 */
	private ContextJungNetwork querySubcontexts(Context context, String name) {
		if (context.getProjection(name) == null) {
			Iterator<Context> contexts = context.getSubContexts().iterator();
			while (contexts.hasNext()) {
				Context c = contexts.next();
				System.out.println("c: " + c.getId() + "/" + name  + "-" + c.getProjections());
				if (c.getProjection(name) != null) {
					return (ContextJungNetwork) c.getProjection(name);
				}
				else {
					querySubcontexts(c, name);
				}
			}
			return null;
		}
		else {
			return (ContextJungNetwork) context.getProjection(name);
		}
	}

	private DefaultDisplayDescriptorAdapted getDescriptor(
			Context context) {
		// queries also subcontexts:
		System.out.println(context.getId());
		NetworkMeasureUtilities.getInstance().addMeasureCalculation(querySubcontexts(context,
				ReSolEvoModel.Network.NETWORK_KNOWN.toString()),
				"IDCnn", new HashMap());
		
		descriptor = new DefaultDisplayDescriptorAdapted<ReSolEvoAgent, ReSolEvoAgent>(
				(Projection<ReSolEvoAgent>) querySubcontexts(context, ReSolEvoModel.Network.NETWORK_KNOWN
								.toString()), context);
		
		descriptor.setDisplayType(DisplayDescriptor.DisplayType.TWO_D);
		descriptor.setLayoutClassName("edu.uos.sh.soneta.layout.KKLayoutImproved2D");
		// update layout every tick:
		descriptor.setLayoutFrequency(LayoutFrequency.AT_INTERVAL);
		descriptor.setLayoutInterval(1);
		
		descriptor.setLayoutProjection(ReSolEvoModel.Network.NETWORK_KNOWN
				.toString());
		descriptor.addNetworkToDisplay((ContextJungNetwork<ReSolEvoAgent>) context
						.getProjection(ReSolEvoModel.Network.NETWORK_KNOWN
								.toString()));
		descriptor.setDisplayNodeTable(true);
		descriptor.addNetworkToDisplay((ContextJungNetwork)context.getProjection(
				ReSolEvoModel.Network.NETWORK_KNOWN.toString()));
		descriptor.addNetworkMeasureToDisplay("IDCnn");
		
		
		return descriptor;
	}
	
	/**
	 * Instantiates a <code>NodeTable</code> for the display this <code>DCCActionAdapted</code>
	 * is associated with and schedules its <code>update()</code> to be executed at PAUSE events.
	 * @date 25.05.2008
	 *
	 */
	protected void setupNodeTable() {
		if (display instanceof Display2DAdapted) {
			AbstractNodeTable nodeTable = new DefaultNodeTable((Display2DAdapted) display, descriptor);
			nodeTable.init();
			logger.debug("Abstract Node Table initialized");
			
			JFrame frame = new JFrame("Node Table");
			frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
			frame.getContentPane().setLayout(new BorderLayout());
			frame.add(nodeTable.getComponent());
			frame.setSize(300, 300);
			frame.pack();
			frame.setVisible(true);
			
			//TODO add multi display support for node tables
//			    //frame.getContentPane().add(this, BorderLayout.CENTER);
//			    
//			    tabbedPane = new JTabbedPane();
//			    
//			    for (DCCActionAdapted<T,S> action : DisplayHandler.<T,S>getInstance().getDccactionList()) {
//			    	if (action.getDescriptor().isDisplayNodeTable()) {
//			    		JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
//			    		splitPane.setContinuousLayout(true);
//			    		splitPane.setLeftComponent(action.getNodeTable().getComponent());
//			    		splitPane.setRightComponent(action.getDisplay().getPanel());
//			    		splitPane.setOneTouchExpandable(true);
//			    		tabbedPane.add(action.getDescriptor().getName(), splitPane);
//			    		
//			    	}
//			    	else {
//			    		tabbedPane.add(action.getDescriptor().getName(), action.getDisplay().getPanel());
//			    	}
//			    }
//			    
//			    frame.getContentPane().add(tabbedPane, BorderLayout.CENTER);
//			    
//			    
//			    
//			    frame.pack();
//			    defaultSize = new Dimension(getWidth(), getHeight());
			
			// TODO scheduling action at pause would be sufficient, but yields inconsistencies:
			RunState.getInstance().getScheduleRegistry().getModelSchedule().schedule(nodeTable);
		}
		else {
			throw new ClassCastException("Tried to create a NodeTable for a display differend from " +
					"Display2DAdapted!");
		}
	}

}