/**
 * This file is part of
 * 
 * Replicating Solidarity Evolution (ReSolEvo) 0.9
 *
 * Copyright (C) 2010 Center for Environmental Systems Research, Kassel, Germany
 * 
 * ReSolEvo is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * ReSolEvo is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Created by Sascha Holzhauer on 07.01.2010
 */
package de.cesr.resolevo.util;


import java.io.File;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import de.cesr.lara.components.impl.Log4jLogger;
import de.cesr.resolevo.ReSolEvoParameterManager;

/**
 * @author Sascha Holzhauer
 * @date 29.03.2010 
 *
 */
public class ReSolEvoParameterDB implements ReSolEvoParameterManager {
	
	/**
	 * Logger
	 */
	static private Logger logger = Log4jLogger.getLogger(ReSolEvoParameterDB.class);

	Map<Parameter, Object> params = new HashMap<Parameter, Object>();
	// work around - sorry
	private static ReSolEvoParameterXML instance = null;
	private Connection con;
	
	private ReSolEvoParameterDB() {
	}
	
	/**
	 * @see de.cesr.resolevo.ReSolEvoParameterManager#getParameter(de.cesr.resolevo.ReSolEvoParameterManager.Parameter)
	 */
	@Override
	public Object getParameter(Parameter parameter) {
		if (params.containsKey(parameter))  {
			return params.get(parameter);
		}
		else {
			return parameter.getDefaultValue();
		}
	}

	/**
	 * @return
	 * Created by Sascha Holzhauer on 29.03.2010
	 */
	public static ReSolEvoParameterXML getInstance() {
		if (instance == null) {
			instance = ReSolEvoParameterXML.getInstance();
		}
		return instance;
	}
	
	private void readDbSettingsFROMXml() {
		try {
			File file = new File("./DBSettings.xml");
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			DocumentBuilder db = dbf.newDocumentBuilder();
			Document doc = db.parse(file);
			doc.getDocumentElement().normalize();
			
			NodeList nodeLst = doc.getElementsByTagName("db");
			Node fstNode = nodeLst.item(0);
			if (fstNode.getNodeType() == Node.ELEMENT_NODE) {
				Element fstElmnt = (Element) fstNode;
				NodeList fstNmElmntLst = fstElmnt.getElementsByTagName("SQL_LOCATION");
				Element fstNmElmnt = (Element) fstNmElmntLst.item(0);
			    NodeList fstNm = fstNmElmnt.getChildNodes();
			    params.put(Parameter.valueOf(Parameter.class, "SQL_LOCATION"),((Node) fstNm.item(0)).getNodeValue());
			    
			    NodeList dbnameNmElmntLst = fstElmnt.getElementsByTagName("SQL_DBNAME");
				Element dbnameNmElmnt = (Element) dbnameNmElmntLst.item(0);
			    NodeList dbnameNm = dbnameNmElmnt.getChildNodes();
			    params.put(Parameter.valueOf(Parameter.class, "SQL_DBNAME"),((Node) dbnameNm.item(0)).getNodeValue());
			    
			    NodeList userNmElmntLst = fstElmnt.getElementsByTagName("SQL_USER");
				Element userNmElmnt = (Element) userNmElmntLst.item(0);
			    NodeList userNm = userNmElmnt.getChildNodes();
			    params.put(Parameter.valueOf(Parameter.class, "SQL_USER"),((Node) userNm.item(0)).getNodeValue());
			    
			    NodeList pwNmElmntLst = fstElmnt.getElementsByTagName("SQL_PASSWORD");
				Element pwNmElmnt = (Element) pwNmElmntLst.item(0);
			    NodeList pwNm = pwNmElmnt.getChildNodes();
			    params.put(Parameter.valueOf(Parameter.class, "SQL_PASSWORD"),((Node) pwNm.item(0)).getNodeValue());
			    
			    NodeList tbNmElmntLst = fstElmnt.getElementsByTagName("SQL_TBLNAME_PARAMS");
				Element tbNmElmnt = (Element) tbNmElmntLst.item(0);
			    NodeList tbNm = tbNmElmnt.getChildNodes();
			    params.put(Parameter.valueOf(Parameter.class, "SQL_TBLNAME_PARAMS"),((Node) tbNm.item(0)).getNodeValue());
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * @see de.cesr.resolevo.ReSolEvoParameterManager#initParameters()
	 */
	@Override
	public void initParameters() {
		
		// TODO fetch param id from GUI
		
		
		// fetch DB settings from XML-file:
		readDbSettingsFROMXml();
		
		String t1 = (String) getParameter(Parameter.SQL_TBLNAME_PARAMS);

		String sql = "SELECT * FROM `" + t1 + "` AS t1 WHERE t1.id = " + getParameter(Parameter.PARAMS_ID) + ";";
		logger.debug("SQL-statement to fetch params: " + sql);
		
		ResultSet result = connect(sql);
		try {
			result.next();
			for (int i = 1; i <= result.getMetaData().getColumnCount(); i++) {
				try {
					// UNDO
					Object o = result.getObject(i);
					String name = result.getMetaData().getColumnName(i);
				
					params.put(Parameter.valueOf(Parameter.class, result.getMetaData().getColumnName(i)), result.getObject(i));
					logger.info("Parameter " + Parameter.valueOf(Parameter.class, result.getMetaData().getColumnName(i)) + " read from" +
							" database. Value: " + result.getObject(i));
		
				}
				catch(IllegalArgumentException e){
				}
			}
			result.close();
		}
		catch (SQLException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Tries to establish a JDBC Connection to the MySQL database given the
	 * settings provided to the constructor.
	 *
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 * @throws ClassNotFoundException
	 * @throws SQLException
	 */
	protected Connection getConnection() throws InstantiationException,
			IllegalAccessException, ClassNotFoundException, SQLException {
		
		Properties properties = new Properties();
		properties.put("user", getParameter(Parameter.SQL_USER));
		properties.put("password", getParameter(Parameter.SQL_PASSWORD));
		String connectTo = "jdbc:mysql://" + getParameter(Parameter.SQL_LOCATION) + (((String)getParameter(Parameter.SQL_LOCATION)).
				endsWith("/")?"":"/") + getParameter(Parameter.SQL_DBNAME);
		
		// error handling:
		if (getParameter(Parameter.SQL_LOCATION) == null) {
			throw new IllegalArgumentException(
				"Invalid database settings: Invalid database location!");
		}
		if (getParameter(Parameter.SQL_DBNAME) == null) {
			throw new IllegalArgumentException(
				"Invalid database settings: Invalid database name!");
		}
		if (getParameter(Parameter.SQL_USER) == null) {
			throw new IllegalArgumentException(
				"Invalid database settings: Invalid user name!");
		}
		if (getParameter(Parameter.SQL_PASSWORD) == null) {
			throw new IllegalArgumentException(
				"Invalid database settings: Invalid password!");
		}
		
		Class.forName("com.mysql.jdbc.Driver").newInstance();
		logger.info("Connect to DB: " + connectTo);
		return DriverManager.getConnection(connectTo, properties);
	}
	
	/**
	 * @param sql
	 * @return
	 * Created by Sascha Holzhauer on 29.03.2010
	 */
	protected ResultSet connect(String sql) {
		try {
			if (con == null || con.isClosed()) {
				con = getConnection();
			}
			if (sql.startsWith("update:")) {
				con.createStatement().executeUpdate(sql.substring(7));
				return null;
			} else {
				return con.createStatement().executeQuery(sql);
			}
		} catch (SQLException e) {
			e.printStackTrace();
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * 
	 * Created by Sascha Holzhauer on 29.03.2010
	 */
	protected void disconnect() {
		try {
			con.close();
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	@repast.simphony.parameter.Parameter(displayName="Parameter ID", usageName="PARAMS_ID")
	public static void setParameterID(int id) {
		ReSolEvoParameterDB.getInstance().params.put(Parameter.PARAMS_ID, id);
	}
	
	@repast.simphony.parameter.Parameter(displayName="Parameter ID", usageName="PARAMS_ID")
	public static void getParameterID() {
		ReSolEvoParameterDB.getInstance().getParameter(ReSolEvoParameterManager.Parameter.PARAMS_ID);
	}
}
