/**
 * ReSolEvo09
 *
 * Center for Environmental Systems Research, Kassel
 * Created by Sascha Holzhauer on 29.11.2010
 */
package de.cesr.resolevo.util;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import de.cesr.lara.components.impl.Log4jLogger;
import de.cesr.resolevo.ReSolEvoParameterManager;

/**
 * ReSolEvo09
 *
 * @author Sascha Holzhauer
 * @date 29.11.2010 
 *
 */
public class ReSolEvoParameterXML implements ReSolEvoParameterManager {

	/**
	 * Logger
	 */
	static private Logger logger = Log4jLogger.getLogger(ReSolEvoParameterXML.class);
	
	Map<Parameter, Object> params = new HashMap<Parameter, Object>();
	private static ReSolEvoParameterXML instance = null;
	
	/**
	 * @return
	 * Created by Sascha Holzhauer on 29.03.2010
	 */
	public static ReSolEvoParameterXML getInstance() {
		if (instance == null) {
			instance = new ReSolEvoParameterXML();
		}
		return instance;
	}
	
	/**
	 * @see de.cesr.resolevo.ReSolEvoParameterManager#getParameter(de.cesr.resolevo.ReSolEvoParameterManager.Parameter)
	 */
	@Override
	public Object getParameter(Parameter parameter) {
		if (params.containsKey(parameter))  {
			return params.get(parameter);
		}
		else {
			return parameter.getDefaultValue();
		}
	}

	/**
	 * @see de.cesr.resolevo.ReSolEvoParameterManager#initParameters()
	 */
	@Override
	public void initParameters() {
		File file = new File("parameter_1.xml");
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db;
		try {
			db = dbf.newDocumentBuilder();
		
			Document doc = db.parse(file);
			doc.getDocumentElement().normalize();
			
			NodeList nodeLst = doc.getElementsByTagName("parameter");
			for (int i = 0; i < nodeLst.getLength(); i++) {
				Element param = (Element) nodeLst.item(i);
				// get a node list of NAME-nodes (there is actually just 1) from an element param:
				NodeList names = param.getElementsByTagName("NAME");
				// extract first (and only) node:
				Node name = names.item(0);
				// Get child nodes of name node:
				NodeList nameChilds = ((Element) name).getChildNodes();
				// Get single child:
				Node singleName = nameChilds.item(0);
				
				NodeList values = param.getElementsByTagName("VALUE");
				Node value = (Element) values.item(0);
				NodeList valueChilds = ((Element) value).getChildNodes();
				// Get single child:
				Node singleValue = valueChilds.item(0);
			    
				Object v = null;
				
				try {
					if (Parameter.valueOf(Parameter.class, singleName.getNodeValue()).getType() == Integer.class) {
						v = Integer.parseInt(singleValue.getNodeValue());
					}
					else if (Parameter.valueOf(Parameter.class, singleName.getNodeValue()).getType() == Double.class) {
						v = Double.parseDouble(singleValue.getNodeValue());
					}
					else if (Parameter.valueOf(Parameter.class, singleName.getNodeValue()).getType() == Float.class) {
						v = Float.parseFloat(singleValue.getNodeValue());
					}
					else if (Parameter.valueOf(Parameter.class, singleName.getNodeValue()).getType() == Boolean.class) {
						v = Boolean.parseBoolean(singleValue.getNodeValue());
					}
					else {
						v = singleValue.getNodeValue();
					}
				}
				catch (Exception e) {
					logger.warn("Error when parsing parameter " + singleName.getNodeValue());
				}
				params.put(Parameter.valueOf(Parameter.class, singleName.getNodeValue()), v);
			}
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		} catch (SAXException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public static void main(String [] arg) {
		ReSolEvoParameterXML man = ReSolEvoParameterXML.getInstance();
		man.initParameters();
	}
}
